"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const swtc_lib_1 = require("swtc-lib");
exports.Remote = swtc_lib_1.Remote;
exports.Transaction = swtc_lib_1.Transaction;
exports.Wallet = swtc_lib_1.Wallet;
const runtime_core_1 = require("@vue/runtime-core");
const config_1 = __importDefault(require("../../config"));
const chalk_1 = __importDefault(require("chalk"));
const state = setup();
exports.state = state;
function setup() {
    const DEBUG = runtime_core_1.ref(false);
    const LIMIT = runtime_core_1.ref(20);
    const config = runtime_core_1.ref({});
    const server = runtime_core_1.computed(() => config.value.server || "");
    const remote = runtime_core_1.ref({});
    const wsConnected = runtime_core_1.ref(false);
    const interval_detect = runtime_core_1.ref(0);
    const interval_heal = runtime_core_1.ref(0);
    const ledger = runtime_core_1.ref({});
    const logs = runtime_core_1.ref([]);
    const status = runtime_core_1.ref({});
    const rateMap = new Map(); // regular map for performance
    const RATE = runtime_core_1.ref(1000);
    const interval_rate = runtime_core_1.ref(0);
    remote.value = new swtc_lib_1.Remote();
    function funcCleanup() {
        interval_detect.value && clearInterval(interval_detect.value);
        interval_heal.value && clearInterval(interval_heal.value);
        interval_rate.value && clearInterval(interval_rate.value);
    }
    async function funcConfig(options = config_1.default) {
        console.log("... applying configuration");
        config.value = Object.assign({}, config.value, options);
        DEBUG.value = config.value.debug;
        if (config.value.rate) {
            RATE.value = config.value.rate;
        }
        console.log(config.value);
    }
    function funcLogIp(ipaddress) {
        if (!rateMap.has(ipaddress)) {
            rateMap.set(ipaddress, [new Date().getTime()]);
        }
        else {
            rateMap.set(ipaddress, rateMap
                .get(ipaddress)
                .filter(e => e > new Date().getTime() - 5 * 60 * 1000));
            rateMap.get(ipaddress).push(new Date().getTime());
        }
        return rateMap.get(ipaddress).length;
    }
    runtime_core_1.watch(() => ledger.value, (value, old_value) => {
        const { ledger_index, ledger_time } = value;
        DEBUG.value &&
            console.log(chalk_1.default.bold.green(`${JSON.stringify({ ledger_index, ledger_time })}`));
    }, { lazy: true });
    runtime_core_1.watch(() => logs.value, (value, old_value) => console.log(chalk_1.default.green(value[0])), { lazy: true });
    runtime_core_1.watch(() => wsConnected.value, (value, old_value) => {
        console.log(chalk_1.default.green(value));
        if (!value) {
            console.log("starting heal of connection, as scheduled");
        }
    }, { lazy: true });
    runtime_core_1.watch(() => server.value, async (value, old_value) => {
        console.log("config change detected");
        remote.value = value ? new swtc_lib_1.Remote({ server: value }) : new swtc_lib_1.Remote();
        try {
            await remote.value.connectPromise();
            wsConnected.value = true;
            remote.value.on("ledger_closed", data => {
                ledger.value = data;
            });
            console.log("... remote connected");
        }
        catch (e) {
            console.log(chalk_1.default.red(`... remote connection failed to ${server.value}`));
            // console.error(e)
        }
    }, { lazy: true });
    // funcConfig(CONFIG)
    interval_rate.value = setInterval(() => {
        try {
            for (const [ipaddress, counts] of rateMap) {
                const valids = counts.filter(e => e > new Date().getTime() - 5 * 60 * 1000);
                if (valids.length) {
                    rateMap.set(ipaddress, valids);
                }
                else {
                    rateMap.delete(ipaddress);
                    if (DEBUG.value)
                        console.log(chalk_1.default.red(`rate cleaned ${ipaddress}`));
                }
            }
            if (DEBUG.value) {
                console.log(chalk_1.default.red(`rate routine, ${rateMap.size} ips`));
            }
        }
        catch (e) { }
    }, 53000);
    interval_detect.value = setInterval(() => {
        try {
            wsConnected.value = remote.value._server._connected;
            if (ledger.value &&
                new Date().getTime() / 1000 -
                    946684800 -
                    Number(ledger.value.ledger_time) >
                    30) {
                // did not get ledgers for up to 30 seconds
                console.log(chalk_1.default.red(`did not get new ledgers for 30 seconds`));
                console.log(chalk_1.default.red(`check the upstream: ${server.value}`));
                wsConnected.value = false;
            }
        }
        catch (e) { }
    }, 10000);
    interval_heal.value = setInterval(async () => {
        if (!wsConnected.value) {
            console.log(chalk_1.default.red(`cron job to monitor backend connection, once a minute`));
            try {
                await remote.value.connectPromise();
                wsConnected.value = true;
            }
            catch (error) {
                console.log(chalk_1.default.red(`unable to connect ${server.value}, try another one?`));
            }
        }
    }, 60000);
    return {
        DEBUG,
        LIMIT,
        remote,
        config,
        server,
        ledger,
        logs,
        status,
        wsConnected,
        funcConfig,
        funcLogIp,
        funcCleanup,
        rateMap,
        RATE
    };
}
//# sourceMappingURL=index.js.map