"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const rest_1 = require("./rest");
const koa_1 = __importDefault(require("koa"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const koa2_request_log_1 = __importDefault(require("koa2-request-log"));
const koa_robots_txt_1 = __importDefault(require("koa-robots.txt"));
const koa_simple_qs_1 = __importDefault(require("koa-simple-qs"));
const static_files_1 = require("./static-files");
const controller_1 = require("./controller");
exports.controller = controller_1.controller;
const index_1 = require("../store/index");
const swagger_1 = __importDefault(require("./swagger"));
const pkg_1 = require("../pkg");
const regex_pkg = new RegExp("^\\/(v[23])*$");
const web = new koa_1.default();
exports.web = web;
// const logger = new Logger().generate() // default setting
const loggerWithOpts = new koa2_request_log_1.default().generate({
    logColor: "#000",
    stream: process.stdout,
    logFmt: ":method :path :status",
    skip(req, res) {
        return res.status >= 400;
    }
});
// web.use(logger)
web.use(loggerWithOpts);
// swagger document
web.use(swagger_1.default);
// static file support:
web.use(static_files_1.staticRouter().routes());
web.use(koa_robots_txt_1.default([]));
// middleware for ip based rate limit
web.use(async (ctx, next) => {
    if (index_1.state.funcLogIp(ctx.request.ip) < index_1.state.RATE.value) {
        await next();
    }
    else {
        const e = new rest_1.APIError("api:ratelimit", `rate limit ${index_1.state.RATE.value} per 5 minutes, try later`);
        ctx.response.status = 400;
        ctx.response.type = "application/json";
        ctx.response.body = { code: e.code, message: e.message };
    }
});
// middleware check backend store.remote.value.isConnected()
web.use(async (ctx, next) => {
    if (regex_pkg.test(ctx.request.path)) {
        ctx.response.status = 200;
        ctx.response.type = "application/json";
        ctx.response.body = { name: pkg_1.name, version: pkg_1.version, description: pkg_1.description, doc: pkg_1.doc };
    }
    else if (index_1.state.remote.value.isConnected()) {
        await next();
    }
    else {
        const e = new rest_1.APIError("api:backend", "currenly diconnected, wait and try");
        ctx.response.status = 400;
        ctx.response.type = "application/json";
        ctx.response.body = { code: e.code, message: e.message };
    }
});
// middleware use qs to parse query
web.use(koa_simple_qs_1.default({ depth: 1, parameterLimit: 10, allowDots: true }));
// parse request body:
web.use(koa_bodyparser_1.default());
// bind .rest() for ctx:
web.use(rest_1.restify());
// Debug settings store.DEBUG.value
web.use(async (ctx, next) => {
    if (index_1.state.DEBUG.value) {
        console.log(`debug: url = ${ctx.request.url}`);
    }
    await next();
});
// add controller middleware:
web.use(controller_1.controller("controllers", index_1.state.DEBUG.value));
// return apiError if no endpoint found
web.use(async (ctx) => {
    const e = new rest_1.APIError("api:endpoint", "api endpoint not found");
    ctx.response.status = 400;
    ctx.response.type = "application/json";
    ctx.response.body = { code: e.code, message: e.message };
});
//# sourceMappingURL=index.js.map